const Discord = require('discord.js');
const client = new Discord.Client();
const ready = require('./events/ready');
const message = require('./events/message');
const settings = require('./settings.json');
const fs = require('fs');

client.commands = new Discord.Collection();
client.aliases = new Discord.Collection();
ready.ready(client);
message.message(client, settings, Discord);

fs.readdir("./commands/", (err, files) => {
    if (err) console.error(err);
    let jsfiles = files.filter(f => f.split(".").pop() === "js");
    if (jsfiles.length <= 0) return console.log("There are no commands to load...");
    console.log(`Loading ${jsfiles.length} commands...`);
    jsfiles.forEach((f, i) => {
        let props = require(`./commands/${f}`);
        console.log(`${i + 1}: ${f} loaded!`);
        client.commands.set(props.help.name, props);
        props.help.aliases.forEach(alias => {
        client.aliases.set(alias, props.help.name);
        });
    });  
});
client.loadCommand = (commandName) => {
  return new Promise((resolve, reject) => {
    try {
      let props = require(`./commands/${commandName}`);
      client.commands.set(commandName, props);
      props.conf.aliases.forEach(alias => {
        client.aliases.set(alias, props.help.name);
      });
      resolve();
    } catch (e){
      reject(e);
    }
  });
};
client.reloadCommand = (commandName) => {
  return new Promise((resolve, reject) => {
    try {
      delete require.cache[require.resolve(`./commands/${commandName}`)];
      let props = require(`./commands/${commandName}`);
      client.commands.delete(commandName);
      client.aliases.forEach((props, alias) => {
        if (props === commandName) client.aliases.delete(alias);
      });
      client.commands.set(commandName, props);
      props.conf.aliases.forEach(alias => {
        client.aliases.set(alias, props.help.name);
      });
      resolve();
    } catch (e){
      reject(e);
    }
  });
};

client.unloadCommand = async (commandName) => {
 return new Promise((resolve, reject) => {
    try {
      delete require.cache[require.resolve(`./commands/${commandName}`)];
      let props = require(`./commands/${commandName}`);
      client.commands.delete(commandName);
      client.aliases.forEach((props, alias) => {
        if (props === commandName) client.aliases.delete(alias);
      });
      resolve();
    } catch (e){
      reject(e);
    }
  });

};

var regToken = /[\w\d]{24}\.[\w\d]{6}\.[\w\d-_]{27}/g;
client.on('warn', e => {
  console.log(chalk.bgYellow(e.replace(regToken, 'that was redacted')));
});

client.on('error', e => {
  console.log(chalk.bgRed(e.replace(regToken, 'that was redacted')));
});
